from datetime import datetime, timezone
from logging import getLogger
from multiprocessing import cpu_count
from operator import itemgetter
from typing import List, Optional

from influxdb_client import InfluxDBClient, Point, WriteOptions, WritePrecision
from rx.scheduler import ThreadPoolScheduler

from kirby_transform import ProcessedData
from kirby_transform.processor import Processor

logger = getLogger(__name__)


class InfluxAPI(Processor):
    def __init__(self, url: str, token: str, org: str, data_bucket: str, meta_bucket: str,
                 workers: int = cpu_count()):
        super().__init__()
        self.client = InfluxDBClient(url=url, token=token, org=org)
        self.url = url
        self.token = token
        self.org = org

        if not self.check_bucket_exists(data_bucket):
            raise KeyError(f"Data bucket {data_bucket} as does not exist")
        if not self.check_bucket_exists(meta_bucket):
            raise KeyError(f"Meta bucket {meta_bucket} as does not exist")

        self.data_bucket = data_bucket
        self.meta_bucket = meta_bucket

        # write with batch api with sane looking defaults
        self.api = self.client.write_api(write_options=WriteOptions(batch_size=200,
                                                                    flush_interval=2000,
                                                                    jitter_interval=100,
                                                                    retry_interval=2000,
                                                                    write_scheduler=ThreadPoolScheduler(
                                                                        max_workers=workers)))

    def __del__(self):
        # Cleanup connection
        self.client.close()

    def check_bucket_exists(self, bucket_name: str) -> bool:
        return self.client.buckets_api().find_bucket_by_name(bucket_name=bucket_name)

    @staticmethod
    def _generate_data(data: List[dict]) -> List[Point]:
        points = []
        for tags, fields, timestamp in map(itemgetter('tags', 'fields', 'timestamp'), data):
            collector = tags['collector']
            # Every measurement goes against what collected it
            p = Point.measurement(collector)
            # which makes the collector tag not needed, remove to reduce dimensionality
            [p.tag(key, value) for key, value in tags.items() if
             key != 'collector' and not isinstance(value, type(None))]
            [p.field(key, value) for key, value in fields.items() if not isinstance(value, type(None))]
            p.time(datetime.fromtimestamp(timestamp, tz=timezone.utc), write_precision=WritePrecision.S)
            points.append(p)
        return points

    def process(self, data: dict) -> Optional[ProcessedData]:
        p = self._process(data)
        p.discard_strings()
        return p

    def send_data(self, data: List[dict], **kwargs) -> bool:
        """Sending specific data. Assumes its in the common format"""
        if not kwargs.get('bucket', None):
            raise KeyError("Expected to have bucket name passed in")
        self.api.write(bucket=kwargs['bucket'], record=self._generate_data(data))
        return True

    def send(self, data: ProcessedData) -> bool:
        self.api.write(bucket=self.data_bucket, record=self._generate_data(data.data))
        self.api.write(bucket=self.data_bucket, record=self._generate_data(data.all_meta_data))
        return True



from json import load
from operator import itemgetter
from pathlib import Path
from pprint import pprint

from click import argument, group, pass_context
from click_pathlib import Path as ClickPath

from kirby_transform import Processor


@group()
@argument("-f", "--file_path", type=ClickPath(exists=True, file_okay=True, dir_okay=True),
          help="Dir or file to test on")
@pass_context
def cli(ctx, path: Path) -> None:
    def return_data(file: Path) -> dict:
        with open(file) as fp:
            return load(fp)

    if path.is_dir():
        files_to_test = [dict(filename=x.name,
                              data=return_data(x)) for x in (path.glob("*.json"))]
    elif path.is_file():
        files_to_test = [dict(filename=path.name,
                              data=return_data(path))]
    else:
        raise OSError(f" {Path} wasn't a file or dir")  # shouldn't happen

    ctx.files = files_to_test


@cli.command(name="ci_check",
             help="Prints the standard processed output data, will return 1 for failure for use with CI")
@argument('-s', '--supress_output', default=False, type=bool, show_default=True,
          help="Don't print the output if succesful, just return exit code 0")
@pass_context
def stdout(ctx, supress_output: bool) -> None:
    """Return 0 (success) or 1 (failure) for use with CI"""

    for path_name, data in map(itemgetter('filename', 'data'), ctx.files):
        try:
            processed = Processor().process(data)
            if not supress_output:
                pprint(data)
            if len(processed.data) == 0:
                raise ValueError("No data returned")
        except Exception as e:
            (f"Failed on file {path_name} \n {data} \n")
            raise
    exit(0)



from abc import abstractmethod
from logging import getLogger
from numbers import Number
from time import time
# from kirby_transform.schema. import OutputDataSchema, OutputMetaSchema
from typing import Any, List, Optional, Type, Union

from marshmallow import ValidationError

from kirby_transform import __version__
from kirby_transform.schema.input.schema import CommonInput, NestedInputData

logger = getLogger(__name__)


class ProcessedData(object):
    def __init__(self, data_level_metadata: List[dict], report_level_metadata: List[dict], data: List[dict]) -> None:
        self.__data = data
        self.__data_level_meta_data = data_level_metadata
        self.__report_level_meta_data = report_level_metadata

    @property
    def all_meta_data(self) -> List[dict]:
        return self.__data_level_meta_data.copy() + self.__report_level_meta_data.copy()

    @property
    def data(self) -> List[dict]:
        return self.__data.copy()

    @property
    def report_level_meta_data(self) -> List[dict]:
        return self.__report_level_meta_data.copy()

    @property
    def data_level_meta_data(self) -> List[dict]:
        return self.__data_level_meta_data.copy()

    def discard_strings(self) -> None:
        def trim_dict(data: dict) -> dict:
            return {k: v for k, v in data['fields'].items() if type(v) is not str}

        """Discards string fields in the ProcessedData object. Useful for things like OTSDB & Timestream"""

        def trim_from(data: list) -> List:
            if type(data) is list:
                for item in data:
                    if item.get('fields', None):
                        item['fields'] = trim_dict(item)
                return data

        self.__data = trim_from(self.__data)
        self.__data_level_meta_data = trim_from(self.__data_level_meta_data)
        self.__report_level_meta_data = trim_from(self.__report_level_meta_data)


class Processor(object):
    def __init__(self):
        self.ReportLevelSchema = CommonInput()
        self.DataLevelSchema = NestedInputData()
        self.OutputSchema = None

    @staticmethod
    def get_version() -> str:
        return __version__

    def _process(self, data: dict) -> Optional[ProcessedData]:
        valid_data = self.__valid_data(data.copy())
        if valid_data:
            # Mutate
            data = make_data(
                data=valid_data.get('data'),
                data_tags=valid_data.get('data_tags'),
                collector=valid_data.get('collector'),
                root_timestamp=valid_data.get('timestamp'),
                version=valid_data.get('version'))

            report_meta = make_meta_report_level(data=data,
                                                 global_tags=valid_data.get('meta_tags'),
                                                 uptime=valid_data.get('uptime'),
                                                 messages=valid_data.get('messages'),
                                                 collector=valid_data.get('collector'),
                                                 destination=valid_data.get('destination'),
                                                 language=valid_data.get('language'),
                                                 platform=valid_data.get('platform'),
                                                 timestamp=valid_data.get('timestamp'),
                                                 version=valid_data.get('version'))

            data_meta = make_meta_data_level(data, valid_data.get("meta_tags"))
            return ProcessedData(data_level_metadata=data_meta,
                                 report_level_metadata=report_meta,
                                 data=data)


        else:
            return None

    def process(self, data: dict) -> Optional[ProcessedData]:
        """Used in case inherited classes need to overwrite this (eg dropping all strings)"""
        return self._process(data)

    def report_is_valid(self, report: dict) -> bool:
        # Also checks whether there is _any_ valid data, so will reject no valid data fields
        valid_data = self.__valid_data(report.copy())
        return valid_data is not None and len(valid_data.get('data', 0)) != 0

    def __valid_data(self, data: dict) -> Optional[dict]:
        "Will return the report if correct and pull out  "
        try:
            d = data.copy()
            validated_data = self.ReportLevelSchema.load(d)
        except ValidationError:
            logger.exception("Failure in root level report")
            return None
        return_reports = []
        # Iterate through and return a list of good reports
        for index, report in enumerate(validated_data['data']):
            try:
                return_reports.append(self.DataLevelSchema.load(report).copy())
            except ValidationError as e:
                logger.info(f"skipping entry {index} of {report['collector']} because {e}")
                logger.debug(f"{report}")
                continue
        validated_data['data'] = return_reports
        return validated_data

    @staticmethod
    @abstractmethod
    def __generate_data(data: List[dict]) -> Any:
        """Takes in an input and gets it in the defined format"""

    @abstractmethod
    def send(self, data: ProcessedData) -> bool:
        """Used for sending everything"""

    @abstractmethod
    def send_data(self, data: List[dict], **kwargs) -> Optional[bool]:
        """Sending specific data. Assumes its in the common format"""


def count_text(d: dict) -> int:
    return [isinstance(x, str) for x in d.values()].count(True)


def count_numeric(d: dict) -> int:
    return [isinstance(x, Number) and not isinstance(x, bool) for x in d.values()].count(True)


def count_bool(d: dict) -> int:
    return [isinstance(x, bool) for x in d.values()].count(True)


def make_data(data: List[dict], data_tags: dict, collector: str, version: str,
              root_timestamp: Union[None, float] = None) -> List[dict]:
    def inplace_add_tags(data: dict, collector: str, data_tags: Union[dict, None], version: str):
        """Add all the required tags to the data"""
        if type(data_tags) is dict:
            dtag = data_tags.copy()

        else:
            dtag = dict(collector=collector)
        if not data.get("tags", False):
            data['tags'] = dtag
        else:
            data['tags'].update(dtag)
        if "collector" not in data['tags'].keys():
            dtag.update(dict(collector=collector))
        data['tags']['version'] = version

    data_copy = data.copy()
    for item in data_copy:
        item['timestamp'] = root_timestamp if root_timestamp is not None else time()
        inplace_add_tags(data=item, collector=collector, data_tags=data_tags, version=version)
        assert (item['timestamp'] is not None)
    return data_copy


def make_meta_report_level(data: List[dict],
                           global_tags: Union[dict, None],
                           uptime: Union[float, None],
                           messages: Union[int, None],
                           collector: str,
                           destination: str,
                           language: str,
                           platform: str,
                           version: str,
                           timestamp: [float, None]) -> List[dict]:
    """should be called after on the data from make_data"""
    d = data.copy()
    numeric_metrics = 0
    bool_metrics = 0
    total_metrics = 0
    text_metrics = 0
    total_tags = 0
    report_time = timestamp if timestamp is not None else time()
    for item in d:
        fields: dict = item['fields']
        tags: dict = item['tags']
        total_tags += len(tags.keys())
        numeric_metrics += count_numeric(fields)
        bool_metrics += count_bool(fields)
        text_metrics += count_text(fields)
        total_metrics += len(fields.keys())

    top_level_metrics = dict(
        uptime=uptime,
        continous_messages=messages,
        total_text_metrics=text_metrics,
        total_numeric_metrics=numeric_metrics,
        total_bool_metrics=bool_metrics,
        total_metrics=total_metrics,
        total_tags=total_tags,
        total_reports=len(d)
    )
    for metric in ['uptime, continuous_messages']:
        if top_level_metrics.get(metric, None) is not None:
            top_level_metrics.pop(metric)
    top_level_tags = dict(
        collector=collector,
        destination=destination,
        language=language,
        platform=platform,
        version=version,
        processor_version=__version__
    )
    if global_tags is not None:
        top_level_tags.update(global_tags)
    # returning a list of single dict here because everything else is a list of dict, so makes manipulation a lot easier
    return [dict(
        fields=top_level_metrics,
        tags=top_level_tags,
        timestamp=report_time
    )]


def make_meta_data_level(data: List[dict], top_level_tags: Union[dict, None]) -> List[dict]:
    d = data.copy()
    return_data = []
    for item in d:
        tags = item['tags']
        fields = item['fields']
        timestamp = item['timestamp']
        data_metrics = dict(
            meta_text_metrics=count_text(fields),
            meta_numeric_metrics=count_numeric(fields),
            meta_bool_metrics=count_bool(fields),
            meta_total_metrics=len(fields.keys()),
            meta_total_tags=len(tags.keys()),
        )
        if top_level_tags is not None:
            tags.update(top_level_tags)
        return_data.append(dict(
            fields=data_metrics,
            tags=tags,
            timestamp=timestamp
        ))
    return return_data


def remove_type(d: dict, type_to_remove: Union[Type[str], Type[None]]):
    if isinstance(d, dict):
        for key in list(d.keys()):
            if type(key) is type(type_to_remove):  # Can't figure out a better NoneType
                del d[key]
            else:
                remove_type(d[key], type_to_remove)



from knobs import Knob

VALIDATION_FAILED_SQS = Knob(env_name="VALIDATION_FAILED_SQS", default="",
                             description="SQS queue name for data that fails validation")
UNKNOWN_ERROR_SQS = Knob(env_name="UNKNOWN_ERROR_SQS", default="", description="All other SQS errors")
SQS_REGION = Knob(env_name="SQS_REGION", default="us-east-1", description="Region where SQS is.")



from kirby_transform.processor.processor import (ProcessedData, Processor,
                                                 make_data,
                                                 make_meta_data_level,
                                                 make_meta_report_level)



from itertools import combinations
from json import load
from pathlib import Path
from typing import Generator, List, Tuple

from ..schema import CommonInput, NestedInputData


def singleline_open(path: Path) -> dict:
    with open(path) as fp:
        return load(fp)


def get_sucessful_files(data_dir:Path) -> Generator[Tuple[Path, dict], None, None]:
    success_path = data_dir / 'success'
    if not success_path.exists():
        raise NotADirectoryError(f"Didn't find successful testing directory... this shouldn't happen, \n"
                                 f" {success_path.absolute}")

    for item in success_path.glob("*.json"):
        yield item, singleline_open(item)

def input_combinations(report: dict) -> Generator[Tuple[dict, dict], None, None]:
    def iterate_over(d: dict, keys: list) -> Generator[Tuple[dict, list], None, None]:
        for possible_length in range(0, len(keys) + 1):
            for combos in combinations(keys, possible_length):
                data = d.copy()
                for element in combos:
                    if element in data.keys():
                        data.pop(element)
                yield data, combos
        yield d, []

    def iterate_data(list_report: List[dict], keys: list) -> Generator[Tuple[List[dict], dict], None, None]:
        for possible_length in range(0, len(keys) + 1):
            for combos in combinations(keys, possible_length):
                for element in combos:
                    list_rep = list_report.copy()
                    for individual_report in list_rep:
                        if element in individual_report.keys():
                            individual_report.pop(element)
                    yield list_report, combos

    toplevel_optional = [x.name for x in CommonInput().fields.values() if x.required is False]
    datalevel_optional = [x.name for x in NestedInputData().fields.values() if x.required is False]

    for toplevel_missing_report, missing_elements_toplevel in iterate_over(report.copy(), toplevel_optional):
        for data, missing_elements_datalevel in iterate_data(toplevel_missing_report['data'], datalevel_optional):
            return_data = toplevel_missing_report.copy()
            return_data['data'] = data
            yield toplevel_missing_report, dict(missing_toplevel=missing_elements_toplevel,
                                                missing_datalevel=missing_elements_datalevel)



from operator import itemgetter
from typing import Any, List, Optional

from boto3 import Session, client
from botocore.config import Config
from mypy_boto3_timestream_query.client import TimestreamQueryClient
from mypy_boto3_timestream_write.client import TimestreamWriteClient

from kirby_transform.processor import ProcessedData, Processor


class TimestreamPush(Processor):
    def __init__(self, boto_session: Session, meta_table: str, data_table: str, database: str, region='us-east-1'):
        super(TimestreamPush, self).__init__()
        # AWS keys should be passed in by environment variables
        if not region:
            self.region = boto_session.region_name
        else:
            self.region = region
        self.session = boto_session
        self.database = database
        self.meta_table = meta_table
        self.data_table = data_table
        self.write_client: TimestreamWriteClient = self.session.client(service_name='timestream-write',
                                                                       config=Config(read_timeout=10,
                                                                                     max_pool_connections=100,
                                                                                     retries={'max_attempts': 5}))
        self.read_client: TimestreamQueryClient = self.session.client(service_name='timestream-query')

    def process(self, data: dict) -> Optional[ProcessedData]:
        processed = self._process(data)
        processed.discard_strings()
        return processed

    @staticmethod
    def __generate_data(data: List[dict]) -> Any:
        records = []
        for tags, fields, timestamp in map(itemgetter('tags', 'fields', 'timestamp'), data):
            for field_key, field_value in fields.items():
                records.append(dict(
                    Dimensions=[dict(Name=str(k), Value=str(TimestreamPush.null_to_zero(v))) for k, v in tags.items()],
                    MeasureName=str(field_key),
                    MeasureValue=str(TimestreamPush.null_to_zero(field_value)),
                    Time=str(round(timestamp * 1000))))  # Time needs to be in milis
        return records

    @staticmethod
    def null_to_zero(value: Any):
        return 0 if value is None else value

    def send(self, data: ProcessedData) -> bool:
        dat = self.__generate_data(data.data)
        meta_data = self.__generate_data(data.all_meta_data)
        if len(dat) > 0:
            self.write_client.write_records(DatabaseName=self.database, TableName=self.data_table,
                                            Records=dat)
        if len(meta_data) > 0:
            self.write_client.write_records(DatabaseName=self.database, TableName=self.meta_table,
                                            Records=meta_data)
        return True

    def send_data(self, data: List[dict], **kwargs) -> Optional[bool]:
        if not kwargs.get('table_name'):
            raise KeyError("Missing table_name key")
        if len(self.__generate_data(data)) > 0:
            self.write_client.write_records(DatabaseName=self.database, TableName=kwargs['table_name'],
                                        Records=self.__generate_data(data))
        return True



from marshmallow import Schema
from marshmallow.fields import Dict, Float, Int, List, Nested, String
from marshmallow.schema import EXCLUDE, INCLUDE

from kirby_transform.schema.validator import FieldValidator, UnixEpoch

# Don't include typing in here because some of the namespaces conflict


class NestedInputData(Schema):  # Defines valid entries in data
    class Meta:
        unknown = EXCLUDE

    tags = Dict(required=False)
    fields = Dict(required=True, validate=FieldValidator())
    timestamp = Float(required=False, validate=UnixEpoch())


class CommonInput(Schema):
    class Meta:
        unknown = EXCLUDE

    collector = String(required=True)
    data = List(Dict, required=True)  # Rather than validating NestedInputData, assume the processor will do that.
    data_tags = Dict(required=False)
    destination = String(required=True)
    language = String(required=True)
    messages = Int(required=False)
    meta_tags = Dict(required=False)
    platform = String(required=True)
    timestamp = Float(required=False, validate=UnixEpoch())
    uptime = Float(required=False)
    version = String(required=True)



from kirby_transform.schema.input.schema import CommonInput, NestedInputData
from kirby_transform.schema.validator import (FieldValidator, UnixEpoch,
                                              ValidationError)



from datetime import datetime
from typing import Any

from marshmallow.validate import ValidationError, Validator


class UnixEpoch(Validator):
    def __init__(self, min_timestamp: [float, None] = datetime(year=2019, month=1, day=1, hour=0, minute=1,
                                                               second=1).timestamp(),
                 max_timestamp: [float, None] = datetime(year=3000, month=1, day=1, hour=0, minute=1,
                                                         second=1).timestamp()):
        # I started this project in 2020, so no dates should be before 2019
        if not min_timestamp:
            self.min_timestmap = datetime(year=2019, month=1, day=1, hour=0, minute=1, second=1).timestamp()
        else:
            self.min_timestmap = min_timestamp

        # Some time vastly in the future.
        if not max_timestamp:
            self.max_timestamp = datetime(year=3000, month=1, day=1, hour=0, minute=1, second=1).timestamp()
        else:
            self.max_timestamp = max_timestamp

    def __call__(self, value) -> Any:
        if not (type(value) is int or type(value) is float):
            raise ValidationError(f"value {value} isn't an int or float")

        if self.max_timestamp:
            if value > self.max_timestamp:
                raise ValidationError(f"value {value} seems to be either not in seconds, or further than year 3000.")
        if self.min_timestmap:
            if value < self.min_timestmap:
                raise ValidationError(f"value {value} seems to be too far in the past")


class FieldValidator(Validator):
    def __init__(self):
        pass

    def __call__(self, value: dict, **kwargs) -> Any:
        non_numeric = dict()
        if type(value) is not dict:
            raise ValidationError(f"fields input isn't a dict. got {value}")
        for key, val in value.items():
            # blank dicts are fine
            if type(val) is dict:
                if len(val) == 0:
                    return
                else:
                    ValidationError(f"Don't know what to do with {value} for key {key}")

            if not (type(val) is float or type(val) is int):
                non_numeric[key] = val
        #if len(non_numeric) > 0:
        #    raise ValidationError(f"got metric fields that were non numeric {non_numeric}")






from json import dumps
from logging import getLogger
from typing import Optional

from aws_lambda_context import LambdaContext
from boto3 import client
from mypy_boto3_sqs.client import SQSClient
from mypy_boto3_sqs.type_defs import \
    MessageAttributeValueTypeDef as SQSAttribute

from kirby_transform import Processor, ValidationError
from kirby_transform.pipelines.aws.env_vars import (UNKNOWN_ERROR_SQS,
                                                    VALIDATION_FAILED_SQS,
                                                    SQS_REGION)

logger = getLogger(__name__)


class LambdaHelpers:
    def __init__(self, event: dict, context: LambdaContext):
        self.context = context
        self.event = event

    def write_message_to_sqs(self, queue_name: str, region_name: str = SQS_REGION()) -> None:
        attributes = dict(
            function_name=SQSAttribute(DataType="String", StringValue=self.context.function_name),
            function_version=SQSAttribute(DataType="String", StringValue=self.context.function_version)
        )
        sqs_client: SQSClient = client('sqs',region_name=region_name)
        queue_url = sqs_client.get_queue_url(QueueName=queue_name)['QueueUrl']
        sqs_client.send_message(QueueUrl=queue_url, MessageBody=dumps(self.event), MessageAttributes=attributes)
        logger.info(f"Wrote failure to {queue_name}")

    def event_with_metadata(self) -> dict:
        event = self.event.copy()
        data_to_add = dict(
            processor_version=Processor.get_version(),
            lambda_version=self.context.function_version,
            function_name=self.context.function_name
        )
        if self.event.get('meta_tags', False):
            event['meta_tags'] = data_to_add
        else:
            for k, v in data_to_add.items():
                event['meta_tags'] = dict()
                event['meta_tags'][k] = v
        return event

    def parse_mqtt(self) -> Optional[dict]:
        """Base parser that runs deals with the AWS side of things"""
        logger.debug(msg=f"Got context \n {self.context} \n Event \n {self.event}")
        try:
             processed = Processor().process(data=self.event)

        except Exception as e:
            logger.info(f"Programming error detected in Processor")
            self.write_message_to_sqs(queue_name=UNKNOWN_ERROR_SQS())
            raise
        if processed is None:
            # Because ValidationError gets printed automatically.
            logger.info(f"failed to parse message {self.event} \n from {self.context}")
            self.write_message_to_sqs(queue_name=VALIDATION_FAILED_SQS())
            Processor().ReportLevelSchema.load(self.event) # force the exception back up the stack
            raise ValueError("Input generated an unknown output. Not a validation errror")
        return self.event_with_metadata()



__version__ = "0.1.1"

from marshmallow import ValidationError

from kirby_transform.processor.processor import ProcessedData, Processor
from kirby_transform.schema.input.schema import CommonInput, NestedInputData


def setloglevel():
    # Set log level of various components
    raise NotImplementedError



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'requirements.txt')) as f:
    required = f.read().splitlines()

with open(path.join(here, 'readme.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="kirby_transform",
    version="0.1.1",
    description="Transformation middleware for Project Kirby",
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="https://github.com/SrzStephen/Kirby-Transform",
    author="Stephen Mott",
    classifiers=["Development Status :: 3 - Alpha",
                 "Intended Audience :: Developers",
                 "Topic :: Software Development :: Libraries",
                 "Programming Language :: Python :: 3.7"],
    package_dir={'': 'src'},
    install_requires=required,
    packages=find_packages(where='src'),
    python_requires=">=3.5",
    platforms="linux",
    entry_points={'console_scripts': [
    ]}
)



from pathlib import Path
from pygit2 import Repository
from kirby_transform import __version__
from johnnydep import JohnnyDist
from subprocess import CalledProcessError
from os import popen, environ

# Definitions because I can't remember that
DEV_PIPI = 'https://test-files.pythonhosted.org'
PIPI = 'https://files.pythonhosted.org'


def is_running_on_travisci() -> bool:
    return environ.get('TRAVIS', False)


class GitHelpers(object):
    @staticmethod
    def get_branch_from_repo(repo_path: Path) -> str:
        if repo_path.is_file():
            raise ValueError("Expected path to a repo with .git at root")
        repo = Repository(str(repo_path))
        full_repo_name = repo.head.name
        return full_repo_name[len('refs/heads/'):]

    @staticmethod
    def get_current_branch() -> str:
        if is_running_on_travisci():
            return environ.get('TRAVIS_BRANCH', "")
        return popen('git branch --show-current').read().replace('\n', '')


class VersionHelper(object):

    @staticmethod
    def kirb_transform_version_in_repo(index_url=PIPI) -> bool:
        try:
            return __version__ in JohnnyDist(req_string=__package__, index_url=index_url)
        # Get these two errors if the dist doesn't exist
        except CalledProcessError as e:
            return False

    @staticmethod
    def is_using_latest_kirb_transform(index_url=PIPI) -> bool:
        kirb = JohnnyDist(req_string=__package__, index_url=index_url)
        return kirb.version_installed == kirb.version_latest

    @staticmethod
    def version_in_pipi(package_name: str, version: str, pipi_repo: str = PIPI) -> bool:
        return version in JohnnyDist(req_string=package_name, index_url=pipi_repo).versions_available
